@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use 5.008001;
 use parent qw/Exporter/;
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 use HTML::TreeBuilder::XPath;
 use LWP::UserAgent;
 use HTML::Selector::XPath 0.06 qw/selector_to_xpath/;
@@ -97,13 +97,13 @@ sub parent {
 
 sub first {
     my $self = shift;
-    $self->{trees} = +[$self->{trees}[0]];
+    $self->{trees} = +[$self->{trees}[0] || ()];
     return $self;
 }
 
 sub last {
     my $self = shift;
-    $self->{trees} = +[$self->{trees}[-1]];
+    $self->{trees} = +[$self->{trees}[-1] || ()];
     return $self;
 }
 
@@ -156,6 +156,34 @@ sub each {
     return $self;
 }
 
+sub map {
+    my ($self, $code) = @_;
+    my $i = 0; 
+    return +[map {
+        my $tree = $_;
+        local $_ = wq($tree);
+        $code->($i++, $_);
+    } @{$self->{trees}}];
+}   
+
+sub filter {
+    my $self = shift;
+
+    if (ref($_[0]) eq 'CODE') {
+        my $code = $_[0];
+        my $i = 0; 
+        $self->{trees} = +[grep {
+            my $tree = $_;
+            local $_ = wq($tree);
+            $code->($i++, $_);
+        } @{$self->{trees}}];
+        return $self;
+
+    } else {
+        return $self->find($_[0])
+    }
+}
+
 sub DESTROY {
     if ($_[0]->{need_delete}) {
         $_->delete for @{$_[0]->{trees}}; # avoid memory leaks
@@ -259,6 +287,14 @@ This method find nodes by $selector from $q. $selector is a CSS3 selector.
 Visit each nodes. C<< $i >> is a counter value, 0 origin. C<< $elem >> is iteration item.
 C<< $_ >> is localized by C<< $elem >>.
 
+=item $q->map(sub { my ($i, $elem) = @_; ... })
+
+Creates a new array with the results of calling a provided function on every element.
+
+=item $q->filter(sub { my ($i, $elem) = @_; ... })
+
+Reduce the elements to those that pass the function's test.
+
 =item $q->end()
 
 Back to the before context like jQuery.
